"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const metadata_1 = require("./interface/metadata");
function findEndpoint(endpoints, funcName) {
    return _.find(endpoints, (endpoint) => endpoint.funcName === funcName);
}
var Decorator;
(function (Decorator) {
    let Event;
    (function (Event) {
        function worker(eventClass) {
            return function (target, propertyKey, descriptor) {
                const endpoints = Reflect.getOwnMetadata(metadata_1.Metadata.Key.WorkerEvent, target.constructor) || [];
                let eventKey;
                if (!_.isString(eventClass)) {
                    const event = new eventClass();
                    eventKey = event.key;
                }
                else {
                    eventKey = eventClass;
                }
                const endpoint = {
                    key: eventKey,
                    funcName: propertyKey
                };
                endpoints.push(endpoint);
                Reflect.defineMetadata(metadata_1.Metadata.Key.WorkerEvent, endpoints, target.constructor);
            };
        }
        Event.worker = worker;
        function fanout(eventClass) {
            return function (target, propertyKey, descriptor) {
                const endpoints = Reflect.getOwnMetadata(metadata_1.Metadata.Key.FanoutEvent, target.constructor) || [];
                let eventKey;
                if (!_.isString(eventClass)) {
                    const event = new eventClass();
                    eventKey = event.key;
                }
                else {
                    eventKey = eventClass;
                }
                const endpoint = {
                    key: eventKey,
                    funcName: propertyKey
                };
                endpoints.push(endpoint);
                Reflect.defineMetadata(metadata_1.Metadata.Key.FanoutEvent, endpoints, target.constructor);
            };
        }
        Event.fanout = fanout;
    })(Event = Decorator.Event || (Decorator.Event = {}));
    let Endpoint;
    (function (Endpoint) {
        function rpc(context) {
            return function (target, propertyKey, descriptor) {
                const endpoints = Reflect.getOwnMetadata(metadata_1.Metadata.Key.Rpc, target.constructor) || [];
                const endpoint = {
                    funcName: propertyKey,
                    context: context || {}
                };
                endpoints.push(endpoint);
                Reflect.defineMetadata(metadata_1.Metadata.Key.Rpc, endpoints, target.constructor);
            };
        }
        Endpoint.rpc = rpc;
        function graph(context) {
            return function (target, propertyKey, descriptor) {
                const endpoints = Reflect.getOwnMetadata(metadata_1.Metadata.Key.Graph, target.constructor) || [];
                const endpoint = {
                    funcName: propertyKey,
                    context: context || {}
                };
                endpoints.push(endpoint);
                Reflect.defineMetadata(metadata_1.Metadata.Key.Graph, endpoints, target.constructor);
            };
        }
        Endpoint.graph = graph;
        function rest(route, context) {
            return function (target, propertyKey, descriptor) {
                const endpoints = Reflect.getOwnMetadata(metadata_1.Metadata.Key.Rest, target.constructor) || [];
                const [method, uri] = route.split(' ');
                const endpoint = findEndpoint(endpoints, propertyKey);
                if (endpoint) {
                    endpoint.method = method;
                    endpoint.uri = uri;
                }
                else {
                    const endpoint = {
                        method: method,
                        uri: uri,
                        funcName: propertyKey,
                        context: context || {}
                    };
                    endpoints.push(endpoint);
                }
                Reflect.defineMetadata(metadata_1.Metadata.Key.Rest, endpoints, target.constructor);
            };
        }
        Endpoint.rest = rest;
    })(Endpoint = Decorator.Endpoint || (Decorator.Endpoint = {}));
    function requireLevel(level) {
        return function (target, propertyKey, descriptor) {
            const endpoints = Reflect.getOwnMetadata(metadata_1.Metadata.Key.Rest, target.constructor) || [];
            const endpoint = findEndpoint(endpoints, propertyKey);
            if (endpoint) {
                const context = endpoint.context || {};
                context.level = level;
                endpoint.context = context;
            }
            else {
                const endpoint = {
                    method: '',
                    uri: '',
                    funcName: propertyKey,
                    context: { level }
                };
                endpoints.push(endpoint);
            }
            Reflect.defineMetadata(metadata_1.Metadata.Key.Rest, endpoints, target.constructor);
        };
    }
    Decorator.requireLevel = requireLevel;
})(Decorator = exports.Decorator || (exports.Decorator = {}));
//# sourceMappingURL=decorator.js.map