"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var StarError;
(function (StarError) {
    /*
      1 0 0 1 0 0 0 1
      _____ _ _______
      |     | \_ errorCode
      |     \_ errorLevel
      \_ starCode
    */
    class AbstractError extends Error {
        constructor(starCode, errorLevel, errorCode, reason = '') {
            const code = AbstractError.mergeCode(starCode, errorLevel, errorCode);
            super(`${code}-${reason}`);
            this.name = 'AbstractError';
            this.code = code;
            this.reason = reason;
            this.extra = {};
        }
        static mergeCode(starCode, errorLevel, errorCode) {
            return starCode * 100000 + errorLevel * 10000 + errorCode;
        }
        toCode(errorLevel, errorCode) {
            return this.code;
        }
    }
    StarError.AbstractError = AbstractError;
    class AbstractExpectedError extends AbstractError {
        constructor(starCode, errorCode, reason) {
            super(starCode, ErrorLevel.Expected, errorCode, reason);
            this.name = 'ExpectedError';
        }
    }
    StarError.AbstractExpectedError = AbstractExpectedError;
    class AbstractLogicError extends AbstractError {
        constructor(starCode, errorCode, reason) {
            super(starCode, ErrorLevel.Logic, errorCode, reason);
            this.name = 'LogicError';
        }
    }
    StarError.AbstractLogicError = AbstractLogicError;
    class AbstractFatalError extends AbstractError {
        constructor(starCode, errorCode, reason) {
            super(starCode, ErrorLevel.Fatal, errorCode, reason);
            this.name = 'FatalError';
        }
    }
    StarError.AbstractFatalError = AbstractFatalError;
    let ErrorLevel;
    (function (ErrorLevel) {
        ErrorLevel[ErrorLevel["Expected"] = 1] = "Expected";
        ErrorLevel[ErrorLevel["Logic"] = 2] = "Logic";
        ErrorLevel[ErrorLevel["Fatal"] = 3] = "Fatal";
    })(ErrorLevel = StarError.ErrorLevel || (StarError.ErrorLevel = {}));
})(StarError = exports.StarError || (exports.StarError = {}));
//# sourceMappingURL=error.js.map